@@ -20,31 +20,32 @@
 typedef struct {
   /* Application widget, and its subwidgets */
   PanelApplet *applet;
-  GtkWidget *button, *playpause, *stop;
-  GtkWidget *button_pixmap, *playpause_pixmap, *stop_pixmap;
+/*  GConfClient *gconf_client;*/
+  GtkWidget *penguin, *playpause, *stop;
+  GtkWidget *penguin_pixmap, *playpause_pixmap, *stop_pixmap;
   GtkWidget *hbox, *vbox;
   /* Property box widgets */
   GtkWidget *prop_window;
   GtkAdjustment *number_adj, *speed_adj;
   GtkWidget *ignorepopups_check;
   GtkWidget *noblood_check, *noangels_check;
+  GtkWidget *squish_check;
   GtkWidget *overridetheme_check;
   GtkWidget *table;
   GtkWidget *preview_pixmap, *preview, *info;
   GtkTreeView *treeview;
   gulong treeview_signal_id;
-  /* Other widgets */
-  GtkWidget *error_dialog;
   /* Theme information */
   gchar **theme_list;
   gchar **theme_info;
-  gchar *theme_name;
+  gchar *theme_name; /* slash-separated list of the active themes */
+  gchar *focussed_theme_name; /* The single selected themes */
   gint nthemes;
   XPenguinsTheme theme;
   /* Properties */
   gint timeout, number, speed;
   gboolean noblood, noangels, overridetheme, ignorepopups;
-  gboolean active, paused, loaded;
+  gboolean active, paused, loaded, squish;
 } XPenguinsApplet;
 
 
