@@ -17,9 +17,9 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Robin Hogan <R.J.Hogan@reading.ac.uk>
- * Sebastian Henschel <shensche@kodeaffe.de>
+ * Sebastian Henschel <xpenguins@kodeaffe.de>
  * Project started 22 November 1999
- * This file last updated 15 January 2003
+ * This file last updated 08 August 2004
  */
 
 /*****************************************************************************
@@ -282,7 +282,7 @@ about (BonoboUIComponent *uic, XPenguinsApplet *xpa, const gchar *verbname)
     static GdkPixbuf *pixbuf;
     static const gchar *authors[] = {
         "Robin Hogan <R.J.Hogan@reading.ac.uk>",
-        "Sebastian Henschel <shensche@kodeaffe.de>",
+        "Sebastian Henschel <xpenguins@kodeaffe.de>",
         NULL
     };
     gchar *translators = g_ascii_strcasecmp (
