@@ -1,6 +1,5 @@
 /* xpenguins_theme.c - provides the themability of xpenguins
  * Copyright (C) 1999-2001  Robin Hogan
- * Copyright (C) 2002       Sebastian Henschel
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,7 +37,7 @@ static
 void
 __xpenguins_out_of_memory(int line)
 {
-  fprintf(stderr, "%s: Out of memory at line %d\n", __FILE__, line);
+  fprintf(stderr, _("%s: Out of memory at line %d\n"), __FILE__, line);
 }
 
 #define xpenguins_out_of_memory() __xpenguins_out_of_memory(__LINE__)
@@ -137,15 +136,16 @@ xpenguins_remove_underscores(char *name)
   return name;
 }
 
-/* Return a list of names of apparently valid themes - basically the
- * directory names from either the user or the system theme
- * directories that contain a file called `config' are
+/* Return a NULL-terminated list of names of apparently valid themes -
+ * basically the directory names from either the user or the system
+ * theme directories that contain a file called "config" are
  * returned. Underscores in the directory names are converted into
  * spaces, but directory names that already contain spaces are
  * rejected. This is because the install program seems to choke on
  * directory names containing spaces, but theme names containing
  * underscores are ugly. The list should be freed with
- * xpenguins_free_list(). */
+ * xpenguins_free_list(). If nthemes is not NULL then it is used to
+ * return the number of themes that were found */
 char **
 xpenguins_list_themes(int *nthemes)
 {
@@ -284,7 +284,7 @@ xpenguins_list_themes(int *nthemes)
     /* Allocate the array of pointers to point to each theme name */
     char *ch = list;
     int j = 1;
-    list_array = (char **) malloc(n * sizeof(char *));
+    list_array = (char **) malloc((n+1) * sizeof(char *));
     if (!list_array) {
       if (list) {
 	free(list);
@@ -302,7 +302,10 @@ xpenguins_list_themes(int *nthemes)
 	++ch;
       }
     }
-    *nthemes = n;
+    list_array[j] = NULL;
+    if (nthemes) {
+      *nthemes = n;
+    }
     return list_array;
   }
 
@@ -324,7 +327,6 @@ xpenguins_free_list(char **list)
 
 /* Return the full path of the specified theme. Spaces in the theme
  * name are converted into underscores. */
-
 char *
 xpenguins_theme_directory(char *name)
 {
@@ -336,7 +338,6 @@ xpenguins_theme_directory(char *name)
   int string_length = 0, root_length;
   struct stat stat_buf;
 
-
   /* First look in $HOME/.xpenguins/themes for config */
   root_length = strlen(home) + strlen(home_root) + strlen(themes) + 1;
   string_length = root_length + strlen(name) + strlen(config) + 1;
@@ -345,7 +346,6 @@ xpenguins_theme_directory(char *name)
     xpenguins_out_of_memory();
     return NULL;
   }
-
   snprintf(config_path, string_length, "%s%s%s/%s%s",
 	   home, home_root, themes, name, config);
   /* Convert spaces to underscores */
@@ -356,7 +356,6 @@ xpenguins_theme_directory(char *name)
     }
     ++ch;
   }
-
   /* See if theme exists */
   if (stat(config_path, &stat_buf) == 0) {
     config_path[string_length-strlen(config)-1] = '\0';
@@ -385,7 +384,6 @@ xpenguins_theme_directory(char *name)
     }
     ++ch;
   }
-
   /* Look for theme */
   if (stat(config_path, &stat_buf) == 0) {
     config_path[string_length-strlen(config)-1] = '\0';
@@ -413,27 +411,34 @@ __xpenguins_copy_properties(ToonData *src, ToonData *dest)
   return;
 }
 
-/* Load the theme named `name' into the theme structure `theme'.
- * On error, a static error message is returned, on success
+/* Append the theme named "name" to the theme structure "theme". To
+ * initialise the theme, set theme->ngenera to 0 before calling this
+ * function. On error, a static error message is returned, on success
  * NULL (i.e. 0) is returned. */
+static
 char *
-xpenguins_load_theme(char *name, XPenguinsTheme *theme)
+__xpenguins_append_theme(char *name, XPenguinsTheme *theme)
 {
   FILE *config;
   char *file_name = xpenguins_theme_directory(name);
   char *file_base, *xpm_file_name, *tmp_name;
   char word[MAX_STRING_LENGTH];
-  static char out_of_memory[] = "Out of memory";
+  static char *out_of_memory;
   ToonData def; /* The default */
   ToonData duf; /* If toon type is unknown... */
   ToonData *current = &def;
   int i, j, word_length, tmp_int;
   int max_file_name_length = 0;
-  unsigned int genus = 0;
+  unsigned int genus = theme->ngenera;
+  unsigned int first_genus = theme->ngenera;
   char started = 0;
 
+  out_of_memory = _("Out of memory");
+
   /* Find theme */
-  if (!file_name) return "Theme not found or config file not present";
+  if (!file_name) {
+    return _("Theme not found or config file not present");
+  }
 
   /* Open config file */
   max_file_name_length = strlen(file_name) + 1 + MAX_STRING_LENGTH;
@@ -452,12 +457,20 @@ xpenguins_load_theme(char *name, XPenguinsTheme *theme)
   config = fopen(file_name, "r");
   if (!config) {
     free(file_name);
-    return "Unreadable config file";
+    return _("Unreadable config file");
   }
 
-  /* Allocate space for theme */
-  if (__xpenguins_theme_init(theme)) {
-    return out_of_memory;
+  /* Allocate space for theme if this is the first one loaded */
+  if (genus == 0) {
+    if (__xpenguins_theme_init(theme)) {
+      return out_of_memory;
+    }
+  }
+  else {
+    if (__xpenguins_theme_grow(theme)) {
+      xpenguins_free_theme(theme);
+      return out_of_memory;
+    }
   }
 
   /* Set default values */
@@ -481,17 +494,13 @@ xpenguins_load_theme(char *name, XPenguinsTheme *theme)
 #define GET_INT(dest) if (NEXT_WORD() > 0) { dest = atoi(word); }
 #define GET_UINT(dest) \
       if (NEXT_WORD() > 0 && (tmp_int = atoi(word)) >= 0) { dest = tmp_int; }
-#define WARNING(message) if (xpenguins_verbose) fprintf(stderr, "Warning: %s\n", message)
-#define WARNING1(message, string)  if (xpenguins_verbose)fprintf(stderr, "Warning: " message "\n", string)
-#define WARNING2(message, string1, string2) if (xpenguins_verbose) fprintf(stderr, \
-                    "Warning: " message "\n", string1, string2)
+
+#define WARNING if(xpenguins_verbose) fprintf
 
   while(NEXT_WORD() != 0) {
     if (word_length < 0) {
-      if (xpenguins_verbose) {
-	fprintf(stderr, "Warning: %s: `%s...' ignored: longer than %d characters\n",
+      WARNING(stderr, _("Warning: %s: \"%s...\" ignored: longer than %d characters\n"),
 		file_name, word, MAX_STRING_LENGTH);
-      }
       continue;
     }
     /* Define a new genus of toon */
@@ -513,13 +522,11 @@ xpenguins_load_theme(char *name, XPenguinsTheme *theme)
 	}
       }
       else if (word_length < 0) {
-	if (xpenguins_verbose) {
-	  fprintf(stderr, "Warning: %s: `%s...' ignored: longer than %d characters\n",
-		  file_name, word, MAX_STRING_LENGTH);
-	}
+	WARNING(stderr, _("Warning: %s: \"%s...\" ignored: longer than %d characters\n"),
+		file_name, word, MAX_STRING_LENGTH);
       }
       else {
-	WARNING("theme config file ended unexpectedly");
+	WARNING(stderr, _("Warning: theme config file ended unexpectedly\n"));
       }
     }
     /* Set preferred frame delay in milliseconds */
@@ -567,12 +574,12 @@ xpenguins_load_theme(char *name, XPenguinsTheme *theme)
 	  current->conf |= (TOON_INVULNERABLE | TOON_NOBLOCK);
 	}
 	else /* Unknown type! */ {
-	  WARNING1("unknown type `%s': ignoring", word);
+	  WARNING(stderr, _("Warning: unknown type \"%s\": ignoring\n"), word);
 	  current = &duf;
 	}
       }
       else {
-	WARNING("theme config file ended unexpectedly");
+	WARNING(stderr, _("Warning: theme config file ended unexpectedly\n"));
       }
     }
     /* Toon properties */
@@ -587,7 +594,7 @@ xpenguins_load_theme(char *name, XPenguinsTheme *theme)
     else if (WORD_IS("pixmap")) { 
       if (NEXT_WORD() > 0) {
 	if (current == &def) {
-	  WARNING("theme config file may not specify a default pixmap");
+	  WARNING(stderr, _("Warning: theme config file may not specify a default pixmap\n"));
 	}
 	else if (current == &duf) {
 	  /* Don't want to allocate any memory for an unused type */
@@ -606,7 +613,7 @@ xpenguins_load_theme(char *name, XPenguinsTheme *theme)
 	  }
 	  if (current->image) {
 	    /* Pixmap is already defined! */
-	    WARNING1("resetting pixmap to %s", word);
+	    WARNING(stderr, _("Warning: resetting pixmap to %s\n"), word);
 	    if (!current->master) {
 	      /* Free old pixmap if it is not a copy */
 	      XpmFree(current->image);
@@ -618,8 +625,9 @@ xpenguins_load_theme(char *name, XPenguinsTheme *theme)
 	    }
 	  }
 
-	  /* Check if the file has been used before... */
-	  for (igenus = 0; igenus <= genus && new_pixmap; ++igenus) {
+	  /* Check if the file has been used before, but only look in
+             the pixmaps for the current theme... */
+	  for (igenus = first_genus; igenus <= genus && new_pixmap; ++igenus) {
 	    ToonData *data = theme->data[igenus];
 	    for (itype = 0; itype < PENGUIN_NTYPES && new_pixmap; ++itype) {
 	      if (data[itype].filename && !data[itype].master
@@ -649,30 +657,28 @@ xpenguins_load_theme(char *name, XPenguinsTheme *theme)
 	      return out_of_memory;
 	      break;
 	    case XpmOpenFailed:
-	      WARNING1("could not read %s", xpm_file_name);
+	      WARNING(stderr, _("Warning: could not read %s\n"), xpm_file_name);
 	      break;
 	    case XpmFileInvalid:
-	      WARNING1("%s is not a valid xpm file", xpm_file_name);
+	      WARNING(stderr, _("Warning: %s is not a valid xpm file\n"), xpm_file_name);
 	      break;
 	    }
 	  }
 	}	      
       }
       else if (word_length < 0) {
-	if (xpenguins_verbose) {
-	  fprintf(stderr, "Warning: %s: `%s...' ignored: longer than %d characters\n",
+	WARNING(stderr, _("Warning: %s: \"%s...\" ignored: longer than %d characters\n"),
 		  file_name, word, MAX_STRING_LENGTH);
-	}
       }
       else {
-	WARNING("theme config file ended unexpectedly");
+	WARNING(stderr, _("Warning: theme config file ended unexpectedly\n"));
       }
     }
     else if (WORD_IS("number")) {
       GET_UINT(theme->number[genus]); 
     }
     else {
-      WARNING1("%s ignored in config file", word);
+      WARNING(stderr, _("Warning: %s ignored in config file\n"), word);
     }
   }
   fclose(config);
@@ -681,7 +687,7 @@ xpenguins_load_theme(char *name, XPenguinsTheme *theme)
   theme->ngenera = genus+1;
 
   /* Now validate our widths, heights etc. with the size of the image */
-  for (i = 0; i < theme->ngenera; ++i) {
+  for (i = first_genus; i < theme->ngenera; ++i) {
     for (j = 0; j < PENGUIN_NTYPES; ++j) {
       current = theme->data[i] + j;
       if (current->exists) {
@@ -689,28 +695,28 @@ xpenguins_load_theme(char *name, XPenguinsTheme *theme)
 	int values;
 	values = sscanf(current->image[0], "%d %d", &imwidth, &imheight);
 	if (values < 2) {
-	  fprintf(stderr, "Error reading width and height of %s\n",
+	  WARNING(stderr, _("Error reading width and height of %s\n"),
 		  current->filename);
 	  xpenguins_free_theme(theme);
-	  return "Invalid xpm data";
+	  return _("Invalid xpm data");
 	}
 	if (imwidth < current->width * current->nframes) {
 	  if ((current->nframes = imwidth / current->width) < 1) {
 	    xpenguins_free_theme(theme);
-	    return "Width of xpm image too small for even a single frame";
+	    return _("Width of xpm image too small for even a single frame");
 	  }
 	  else {
-	    WARNING1("width of %s is too small to display all the frames",
+	    WARNING(stderr, _("Warning: width of %s is too small to display all the frames\n"),
 		    current->filename);
 	  }
 	}
 	if (imheight < current->height * current->ndirections) {
 	  if ((current->ndirections = imheight / current->height) < 1) {
 	    xpenguins_free_theme(theme);
-	    return "Height of xpm image too small for even a single frame";
+	    return _("Height of xpm image too small for even a single frame");
 	  }
 	  else {
-	    WARNING1("height of %s is too small to display all the frames",
+	    WARNING(stderr, _("Warning: height of %s is too small to display all the frames\n"),
 		    current->filename);
 	  }
 	}
@@ -719,13 +725,12 @@ xpenguins_load_theme(char *name, XPenguinsTheme *theme)
     if ((!theme->data[i][PENGUIN_WALKER].exists)
 	|| (!theme->data[i][PENGUIN_FALLER].exists)) {
       xpenguins_free_theme(theme);
-      return "Theme must contain at least walkers and fallers";
+      return _("Theme must contain at least walkers and fallers");
     }
   }
 
   /* Work out the total number */
-  theme->total = 0;
-  for (i = 0; i < theme->ngenera; ++i) {
+  for (i = first_genus; i < theme->ngenera; ++i) {
     theme->total += theme->number[i];
   }
   if (theme->total > PENGUIN_MAX) {
@@ -738,6 +743,33 @@ xpenguins_load_theme(char *name, XPenguinsTheme *theme)
   return NULL;
 }
 
+/* Load the named theme */
+char *
+xpenguins_load_theme(char *name, XPenguinsTheme *theme)
+{
+  theme->ngenera = 0;
+  theme->total = 0;
+  return __xpenguins_append_theme(name, theme);
+}
+
+/* Load the themes in the string array (where the array is NULL
+ * terminated, as well as all the strings that are pointed to) */
+char *
+xpenguins_load_themes(char **names, XPenguinsTheme *theme)
+{
+  theme->ngenera = 0;
+  theme->total = 0;
+  while (*names) {
+    char *error;
+    error = __xpenguins_append_theme(*names, theme);
+    if (error) {
+      return error;
+    }
+    ++names;
+  }
+  return NULL;
+}
+
 /* Free all the data associated with a theme */
 void
 xpenguins_free_theme(XPenguinsTheme *theme)
@@ -777,7 +809,7 @@ xpenguins_describe_theme(XPenguinsTheme *theme)
 {
   int i, j;
   if (!theme) {
-    fprintf(stderr, "No theme!\n");
+    fprintf(stderr, _("No theme!\n"));
     return;
   }
   for (i = 0; i < theme->ngenera; ++i) {
@@ -787,7 +819,7 @@ xpenguins_describe_theme(XPenguinsTheme *theme)
     for (j = 0; j < PENGUIN_NTYPES; ++j) {
       ToonData *data = theme->data[i] + j;
       if (data->exists) {
-	fprintf(stderr, "  Toon %d: %s %ux%u %ux%u\n", i, data->filename,
+	fprintf(stderr, _("  Toon %d: %s %ux%u %ux%u\n"), i, data->filename,
 		data->nframes, data->ndirections, data->width, data->height);
       }
     }
@@ -808,7 +840,7 @@ xpenguins_describe_theme(XPenguinsTheme *theme)
     }; \
   }
 
-/* Read the `about' file in a theme directory and return a string list
+/* Read the "about" file in a theme directory and return a string list
  * of theme properties. This list should be freed with
  * xpenguins_free_list(). */
 char **
@@ -830,7 +862,7 @@ xpenguins_theme_info(char *name)
   int list_length = 0;
   int offsets[PENGUIN_NABOUTFIELDS];
 
-  /* Determine `about' filename string */
+  /* Determine "about" filename string */
   if (!theme_dir) {
     return NULL;
   }
@@ -849,10 +881,10 @@ xpenguins_theme_info(char *name)
     offsets[i] = 0;
   }
 
-  /* Open `about' file */
+  /* Open "about" file */
   about_file = fopen(file_name, "r");
   if (!about_file) {
-    fprintf(stderr, "Could not read %s\n", file_name);
+    fprintf(stderr, _("Could not read %s\n"), file_name);
     free(file_name);
     free(theme_dir);
     return NULL;
